/*!
 * VisualEditor UserInterface Math icon styles.
 *
 * @copyright 2011-2015 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.oo-ui-icon-math {
	/* @embed */
	background-image: url( math.svg );
}

.oo-ui-icon-math-display-block {
	/* @embed */
	background-image: url( math-display-block.svg );
}

.oo-ui-icon-math-display-default {
	/* @embed */
	background-image: url( math-display-default.svg );
}

.oo-ui-icon-math-display-inline {
	/* @embed */
	background-image: url( math-display-inline.svg );
}
