/*!
 * VisualEditor UserInterface MWMathDialog styles.
 *
 * @copyright 2011-2015 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-ui-mwMathDialog-content .oo-ui-bookletLayout-stackLayout {
	padding: 0.5em;
}

.ve-ui-mwMathDialog-content .oo-ui-bookletLayout-stackLayout h3 {
	margin: 0 0 0.5em 0;
	color: #888;
	font-weight: normal;
	font-size: 1em;
}

.ve-ui-mwMathDialog-content .oo-ui-textInputWidget {
	max-width: none;
}

.ve-ui-mwMathDialog-menuLayout > .oo-ui-menuLayout-content {
	border-bottom: 1px solid #e6e6e6;
	box-shadow: 0 1px 0 0 rgba( 0, 0, 0, 0.05 );
	overflow: auto;
}

.ve-ui-mwMathDialog-menuLayout-short .oo-ui-menuLayout-menu {
	height: 10em;
}

.ve-ui-mwMathDialog-menuLayout-short .oo-ui-menuLayout-content {
	bottom: 10em;
}

.ve-ui-mwMathPage-symbol {
	background-position: center;
	background-repeat: no-repeat;
	cursor: pointer;
	font-size: 1.5em;
	width: 1.8em;
	height: 1.8em;
	display: inline-block;
	vertical-align: middle;
	margin: 0 0.1em 0.1em 0;
	box-sizing: border-box;
	border: 1px solid #e6e6e6;
	-webkit-transition: border-color 200ms;
	-moz-transition: border-color 200ms;
	-o-transition: border-color 200ms;
	transition: border-color 200ms;
}

.ve-ui-mwMathPage-symbol-wide {
	width: 4.65em;
}

.ve-ui-mwMathPage-symbol-contain {
	background-size: contain;
}

.ve-ui-mwMathPage-symbol-largeLayout {
	height: 3.7em;
	width: 4.65em;
}

.ve-ui-mwMathPage-symbol-largeLayout.ve-ui-mwMathPage-symbol-wide {
	width: 9.4em;
}

.ve.ui.mwMathPage-symbol:hover {
	border-color: #ccc;
}

.ve-ui-mwMathDialog-preview {
	text-align: center;
	line-height: 10em;
	height: 10em;
}

.ve-ui-mwMathDialog-menuLayout-short .ve-ui-mwMathDialog-preview {
	line-height: 6em;
	height: 6em;
}

/* Ensures the preview image is vertically centred */
.ve-ui-mwMathDialog-preview .mwe-math-fallback-image-display {
	display: inline-block;
}

.ve-ui-mwMathPage-outline {
	padding: 0.5em;
	color: #555;
}
