/*!
 * VisualEditor UserInterface MWMathInspector styles.
 *
 * @copyright 2011-2015 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-ui-mwMathInspector-content .ve-ui-mwExtensionWindow-input textarea {
	font-family: monospace, Courier;
}
