<?php
/**
 * Aliases for Math
 *
 * In addition to this file, there is Math.alias.noTranslate.php that contains invisible
 * special pages.
 *
 * @file
 * @ingroup Extensions
 */
// @codingStandardsIgnoreFile

$specialPageAliases = [];

/** English (English) */
$specialPageAliases['en'] = [
	'MathStatus' => [ 'MathStatus' ],
];

/** Arabic (العربية) */
$specialPageAliases['ar'] = [
	'MathStatus' => [ 'حالة_الرياضيات' ],
];

/** Persian (فارسی) */
$specialPageAliases['fa'] = [
	'MathStatus' => [ 'وضعیت_ریاضی' ],
];

/** Macedonian (македонски) */
$specialPageAliases['mk'] = [
	'MathStatus' => [ 'МатСтатус' ],
];

/** Simplified Chinese (中文（简体）‎) */
$specialPageAliases['zh-hans'] = [
	'MathStatus' => [ '数学状态' ],
];