/*
 * Document   : ext.math
 * Created on : 23.09.2013, 13:55:00
 * Author     : Physikerwelt (Moritz Schubotz)
 * Description: Shows browser-dependent math output.
 */
@namespace m url('http://www.w3.org/1998/Math/MathML');

/* Default style for MathML. */
.mwe-math-mathml-inline { display: inline !important; }
.mwe-math-mathml-display { display: block !important; margin-left: auto; margin-right: auto; }
.mwe-math-mathml-a11y {
	/* We try to hide the MathML formula in a way that still makes it accessible to accessibility tools. */
	clip: rect(1px, 1px, 1px, 1px);
	overflow: hidden;
	position: absolute;
	width: 1px;
	height: 1px;
	opacity: 0;
}
m|math {
	/* Try some popular OpenType MATH fonts before the WOFF fallback */
	font-family: Cambria Math, Latin Modern Math, STIX Math, LatinModernMathWOFF, serif;
}

/* Default style for the image fallback. */
/* Note: We had to use !important rules because of conflicts with the style
generated by Mathoid. See https://gerrit.wikimedia.org/r/#/c/166213/ */
.mwe-math-fallback-image-inline { display: inline-block; vertical-align: middle; }
.mwe-math-fallback-image-display { display: block; margin-left: auto !important; margin-right: auto !important; }

/* Default style for the source fallback. */
.mwe-math-fallback-source-inline { display: inline; vertical-align: middle; }
.mwe-math-fallback-source-display { display: block; margin-left: auto; margin-right: auto; }

/* PNG related styles from core */
img.tex {
	vertical-align: middle;
}
